@@ -1,13 +1,15 @@
 """Test Home Assistant Stream Deck YAML."""
+
 from __future__ import annotations
 
 import asyncio
+import functools as ft
 import json
 import sys
 import textwrap
 from pathlib import Path
 from typing import Any
-from unittest.mock import Mock, patch
+from unittest.mock import MagicMock, Mock, call, patch
 
 import pytest
 import websockets
@@ -15,6 +17,7 @@ from dotenv import dotenv_values
 from PIL import Image
 from pydantic import ValidationError
 from StreamDeck.Devices.StreamDeckOriginal import StreamDeckOriginal
+from websockets.exceptions import ConnectionClosedError  # noqa: F401
 
 from home_assistant_streamdeck_yaml import (
     ASSETS_PATH,
@@ -39,7 +42,9 @@ from home_assistant_streamdeck_yaml import (
     _to_filename,
     _url_to_filename,
     get_states,
+    run,
     setup_ws,
+    update_all_key_images,
     update_key_image,
 )
 
@@ -48,30 +53,43 @@ sys.path.append(str(ROOT))
 TEST_STATE_FILENAME = ROOT / "tests" / "state.json"
 IS_CONNECTED_TO_HOMEASSISTANT = False
 BUTTONS_PER_PAGE = 15
+DEFAULT_CONFIG_ENCODING = "utf-8"
 
 
 def test_load_config() -> None:
     """Test Config.load."""
-    Config.load(DEFAULT_CONFIG)
+    Config.load(DEFAULT_CONFIG, yaml_encoding=DEFAULT_CONFIG_ENCODING)
 
 
 def test_reload_config() -> None:
     """Test Config.load."""
-    c = Config.load(DEFAULT_CONFIG)
+    c = Config.load(DEFAULT_CONFIG, yaml_encoding=DEFAULT_CONFIG_ENCODING)
     c.pages = []
     assert c.pages == []
     c.reload()
     assert c.pages != []
 
 
-@pytest.fixture()
+def test_load_config_no_pages_raises_error(tmp_path: Path) -> None:
+    """Test that loading a config with no pages raises ValueError.
+
+    Regression test for #280 - previously raised IndexError.
+    """
+    config_file = tmp_path / "empty_config.yaml"
+    config_file.write_text("pages: []")
+
+    with pytest.raises(ValueError, match="No pages defined"):
+        Config.load(config_file, yaml_encoding=DEFAULT_CONFIG_ENCODING)
+
+
+@pytest.fixture
 def state() -> dict[str, dict[str, Any]]:
     """State fixture."""
     with TEST_STATE_FILENAME.open("r") as f:
         return json.load(f)
 
 
-@pytest.fixture()
+@pytest.fixture
 def button_dict() -> dict[str, dict[str, Any]]:
     """Different button configurations."""
     return {
@@ -121,6 +139,7 @@ def button_dict() -> dict[str, dict[str, Any]]:
         "script_with_text_and_icon": {
             "service": "script.turn_off_everything",
             "text": "ALL OFF",
+            "text_offset": 4,
             "icon": "night_sky.png",
         },
         "input_select_with_template": {
@@ -158,7 +177,7 @@ def button_dict() -> dict[str, dict[str, Any]]:
     }
 
 
-@pytest.fixture()
+@pytest.fixture
 def buttons(button_dict: dict[str, dict[str, Any]]) -> list[Button]:
     """List of `Button`s."""
     button_order = [
@@ -183,7 +202,7 @@ def buttons(button_dict: dict[str, dict[str, Any]]) -> list[Button]:
     return [Button(**button_dict[key]) for key in button_order]
 
 
-@pytest.fixture()
+@pytest.fixture
 def config(buttons: list[Button]) -> Config:
     """Config fixture."""
     page_1 = Page(buttons=buttons[:BUTTONS_PER_PAGE], name="Home")
@@ -217,6 +236,17 @@ def test_example_config_browsing_pages(config: Config) -> None:
     assert config.button(0) == first_page.buttons[0]
 
 
+def test_example_close_pages(config: Config) -> None:
+    """Test example config close pages."""
+    assert isinstance(config, Config)
+    assert config._current_page_index == 0
+    second_page = config.next_page()
+    assert isinstance(second_page, Page)
+    assert config._current_page_index == 1
+    config.close_page()
+    assert config._current_page_index == 0
+
+
 @pytest.mark.skipif(
     not IS_CONNECTED_TO_HOMEASSISTANT,
     reason="Not connected to Home Assistant",
@@ -252,9 +282,7 @@ def test_buttons(buttons: list[Button], state: dict[str, dict[str, Any]]) -> Non
     page = Page(name="Home", buttons=buttons)
     config = Config(pages=[page])
     first_page = config.to_page(0)
-    rendered_buttons = []
-    for button in first_page.buttons:
-        rendered_buttons.append(button.rendered_template_button(state))
+    rendered_buttons = [button.rendered_template_button(state) for button in first_page.buttons]
 
     b = rendered_buttons[0]  # LIGHT
     assert b.domain == "light"
@@ -300,6 +328,55 @@ def test_validate_special_type(button_dict: dict[str, dict[str, Any]]) -> None:
         Button(**dict(button_dict["special_goto_0"], special_type_data=[]))
 
 
+def test_long_press_target_allowed() -> None:
+    """Test that 'target' is allowed in long_press configuration.
+
+    Regression test: Previously 'target' was missing from allowed_keys in
+    _validate_long_press, causing ValidationError when using target in long_press.
+    """
+    # This should NOT raise ValidationError - target is a valid key
+    button = Button(
+        service="light.turn_on",
+        long_press={
+            "service": "light.turn_off",
+            "target": {"entity_id": "light.living_room"},
+        },
+    )
+    assert button.long_press is not None
+    assert button.long_press["target"] == {"entity_id": "light.living_room"}
+
+    # Test with all valid long_press keys together
+    button2 = Button(
+        entity_id="light.bedroom",
+        service="light.toggle",
+        long_press={
+            "service": "light.turn_off",
+            "service_data": {"brightness": 50},
+            "entity_id": "light.kitchen",
+            "target": {"area_id": "living_room"},
+            "special_type": "go-to-page",
+            "special_type_data": "settings",
+        },
+    )
+    assert button2.long_press is not None
+    assert button2.long_press["target"] == {"area_id": "living_room"}
+
+
+def test_long_press_target_validation() -> None:
+    """Test that long_press.target must be a dictionary.
+
+    Regression test: Validates that target type checking was added.
+    """
+    with pytest.raises(ValidationError, match=r"long_press\.target must be a dictionary"):
+        Button(
+            service="light.turn_on",
+            long_press={
+                "service": "light.turn_off",
+                "target": "light.living_room",  # Wrong type - should be dict
+            },
+        )
+
+
 def test_download_and_save_mdi() -> None:
     """Test whether function downloads MDI correctly."""
     # might be cached
@@ -329,7 +406,7 @@ def test_init_icon() -> None:
     _init_icon(size=(100, 100))
 
 
-@pytest.fixture()
+@pytest.fixture
 def mock_deck() -> Mock:
     """Mocks a StreamDeck."""
     deck_mock = Mock(spec=StreamDeckOriginal)
@@ -348,6 +425,7 @@ def mock_deck() -> Mock:
     }
 
     deck_mock.key_count.return_value = 15
+    deck_mock.dial_count.return_value = 0
 
     # Add the context manager methods
     deck_mock.__enter__ = Mock(return_value=deck_mock)
@@ -419,9 +497,10 @@ def test_light_page() -> None:
     """Test light page."""
     page = _light_page(
         entity_id="light.bedroom",
-        n_colors=10,
+        n_colors=9,
         colormap="hsv",
         colors=None,
+        color_temp_kelvin=None,
     )
     buttons = page.buttons
     assert len(buttons) == BUTTONS_PER_PAGE
@@ -429,9 +508,10 @@ def test_light_page() -> None:
 
     page = _light_page(
         entity_id="light.bedroom",
-        n_colors=10,
+        n_colors=9,
         colormap=None,
         colors=None,
+        color_temp_kelvin=None,
     )
     buttons = page.buttons
     assert len(buttons) == BUTTONS_PER_PAGE
@@ -452,9 +532,10 @@ def test_light_page() -> None:
 
     page = _light_page(
         entity_id="light.bedroom",
-        n_colors=10,
+        n_colors=9,
         colormap=None,
         colors=hex_colors,
+        color_temp_kelvin=None,
     )
     buttons = page.buttons
 
@@ -494,10 +575,10 @@ def test_generate_uniform_hex_colors() -> None:
     )
 
 
-@pytest.fixture()
+@pytest.fixture
 def websocket_mock() -> Mock:
-    """Mock websocket client protocol."""
-    return Mock(spec=websockets.WebSocketClientProtocol)
+    """Mock websocket client connection."""
+    return Mock(spec=websockets.ClientConnection)
 
 
 async def test_handle_key_press_toggle_light(
@@ -509,7 +590,14 @@ async def test_handle_key_press_toggle_light(
     """Test handle_key_press toggle light."""
     button = config.button(0)
     assert button is not None
-    await _handle_key_press(websocket_mock, state, config, button, mock_deck)
+    await _handle_key_press(
+        websocket_mock,
+        state,
+        config,
+        button,
+        mock_deck,
+        is_long_press=False,
+    )
 
     websocket_mock.send.assert_called_once()
     send_call_args = websocket_mock.send.call_args.args[0]
@@ -530,7 +618,14 @@ async def test_handle_key_press_next_page(
     """Test handle_key_press next page."""
     button = config.button(14)
     assert button is not None
-    await _handle_key_press(websocket_mock, state, config, button, mock_deck)
+    await _handle_key_press(
+        websocket_mock,
+        state,
+        config,
+        button,
+        mock_deck,
+        is_long_press=False,
+    )
 
     # No service should be called
     websocket_mock.send.assert_not_called()
@@ -555,7 +650,14 @@ async def test_button_with_target(
     _button = config.button(0)
     assert _button is not None
     assert _button.service == "media_player.join"
-    await _handle_key_press(websocket_mock, {}, config, _button, mock_deck)
+    await _handle_key_press(
+        websocket_mock,
+        {},
+        config,
+        _button,
+        mock_deck,
+        is_long_press=False,
+    )
     # Check that the send method was called with the correct payload
     called_payload = json.loads(websocket_mock.send.call_args.args[0])
     expected_payload = {
@@ -928,6 +1030,30 @@ async def test_button_with_target(
             {"switch.wifi_switch": {"state": "off"}},
             "wifi-off",
         ),
+        # Test is_number filter
+        (
+            """
+            {% if state_attr('sensor.temp1', 'temperature') | is_number %}
+            {{ state_attr('sensor.temp1', 'temperature') }}°C
+            {% else %}
+            {{ state_attr('sensor.temp1', 'temperature') }}
+            {% endif %}
+            """,
+            {"sensor.temp1": {"attributes": {"temperature": "unavailable"}}},
+            "unavailable",
+        ),
+        # Test is_number filter
+        (
+            """
+            {% if state_attr('sensor.temp1', 'temperature') | is_number %}
+            {{ state_attr('sensor.temp1', 'temperature') }}°C
+            {% else %}
+            {{ state_attr('sensor.temp1', 'temperature') }}
+            {% endif %}
+            """,
+            {"sensor.temp1": {"attributes": {"temperature": 3.2}}},
+            "3.2°C",
+        ),
     ],
 )
 def test_render_jinja2_from_examples_readme(
@@ -1074,6 +1200,170 @@ def test_to_markdown_table() -> None:
     assert isinstance(table, str)
 
 
+async def test_long_press(
+    mock_deck: Mock,
+    websocket_mock: Mock,
+    state: dict[str, dict[str, Any]],
+) -> None:
+    """Test long press."""
+    # Use a higher threshold to account for icon rendering time
+    long_press_threshold = 2.0
+    short_press_time = 0.0
+    assert short_press_time < long_press_threshold
+    long_press_time = long_press_threshold + 0.5
+    assert long_press_time > long_press_threshold
+
+    home = Page(
+        name="home",
+        buttons=[
+            Button(
+                special_type="go-to-page",
+                special_type_data="short",
+                long_press={"special_type": "go-to-page", "special_type_data": "long"},
+            ),
+            Button(special_type="go-to-page", special_type_data="short"),
+        ],
+    )
+    short = Page(
+        name="short",
+        buttons=[
+            Button(text="short", special_type="go-to-page", special_type_data="home"),
+        ],
+    )
+    long = Page(
+        name="long",
+        buttons=[
+            Button(text="long", special_type="go-to-page", special_type_data="home"),
+        ],
+    )
+    config = Config(pages=[home, short, long], long_press_duration=long_press_threshold)
+    assert config._current_page_index == 0
+    assert config.current_page() == home
+
+    press_event = ft.partial(_on_press_callback(websocket_mock, state, config), mock_deck)
+
+    async def press(key: int) -> None:
+        await press_event(key, True)  # noqa: FBT003
+
+    async def release(key: int) -> None:
+        await press_event(key, False)  # noqa: FBT003
+
+    async def press_and_release(key: int, seconds: float) -> None:
+        await press(key)
+        await asyncio.sleep(seconds)
+        await release(key)
+
+    await press_and_release(0, short_press_time)
+    assert config.current_page() == short
+    await press_and_release(0, short_press_time)
+    assert config.current_page() == home
+    await press_and_release(0, long_press_time)
+    assert config.current_page() == long
+    await press_and_release(0, short_press_time)
+    assert config.current_page() == home
+    await press_and_release(1, long_press_time)
+    # uses `short` action because no long action is configured
+    assert config.current_page() == short
+
+    # NOTE: A potential future enhancement would be to trigger the long press action
+    # automatically when the threshold is reached (without waiting for release).
+    # This would require background monitoring and is not currently implemented -
+    # the long press action only triggers on key release.
+
+
+async def test_long_press_template_rendering(
+    mock_deck: Mock,
+    websocket_mock: Mock,
+) -> None:
+    """Test that templates in long_press are rendered before calling service.
+
+    Regression test: Previously, values were extracted from long_press BEFORE
+    rendered_template_button() was called, so templates weren't rendered.
+    """
+    state = {
+        "light.living_room": {"state": "on", "attributes": {"brightness": 200}},
+    }
+    button = Button(
+        entity_id="light.living_room",
+        service="light.turn_on",
+        long_press={
+            "service": "light.turn_on",
+            "service_data": {
+                "entity_id": "light.living_room",
+                # Template that should be rendered to "78" (200 * 100 / 255 ≈ 78)
+                "brightness_pct": '{{ (state_attr("light.living_room", "brightness") * 100 / 255) | int }}',
+            },
+        },
+    )
+    config = Config(pages=[Page(name="test", buttons=[button])])
+
+    await _handle_key_press(
+        websocket_mock,
+        state,
+        config,
+        button,
+        mock_deck,
+        is_long_press=True,
+    )
+
+    # Verify call_service was called
+    websocket_mock.send.assert_called_once()
+    send_call_args = websocket_mock.send.call_args.args[0]
+    payload = json.loads(send_call_args)
+
+    # The template should have been rendered to the actual value
+    assert payload["type"] == "call_service"
+    assert payload["domain"] == "light"
+    assert payload["service"] == "turn_on"
+    # Critical assertion: template must be rendered, not passed as raw string
+    assert payload["service_data"]["brightness_pct"] == "78", (
+        "Template in long_press.service_data was not rendered! "
+        f"Got: {payload['service_data']['brightness_pct']}"
+    )
+
+
+async def test_long_press_service_from_rendered_button(
+    mock_deck: Mock,
+    websocket_mock: Mock,
+) -> None:
+    """Test that long_press.service template is rendered.
+
+    Regression test: Ensures service name templates are also rendered.
+    """
+    state = {
+        "input_select.action": {"state": "turn_off", "attributes": {}},
+    }
+    button = Button(
+        entity_id="light.living_room",
+        service="light.turn_on",
+        long_press={
+            # Template that should render to "light.turn_off"
+            "service": '{{ "light." ~ states("input_select.action") }}',
+            "service_data": {"entity_id": "light.living_room"},
+        },
+    )
+    config = Config(pages=[Page(name="test", buttons=[button])])
+
+    await _handle_key_press(
+        websocket_mock,
+        state,
+        config,
+        button,
+        mock_deck,
+        is_long_press=True,
+    )
+
+    websocket_mock.send.assert_called_once()
+    payload = json.loads(websocket_mock.send.call_args.args[0])
+
+    # The service template should have been rendered
+    assert payload["domain"] == "light"
+    assert payload["service"] == "turn_off", (
+        "Template in long_press.service was not rendered! "
+        f"Got domain.service: {payload['domain']}.{payload['service']}"
+    )
+
+
 async def test_anonymous_page(
     mock_deck: Mock,
     websocket_mock: Mock,
@@ -1086,7 +1376,11 @@ async def test_anonymous_page(
     )
     anon = Page(
         name="anon",
-        buttons=[Button(text="yolo"), Button(text="foo", delay=0.1)],
+        buttons=[
+            Button(text="yolo"),
+            Button(text="foo", delay=0.1),
+            Button(special_type="close-page"),
+        ],
     )
     config = Config(pages=[home], anonymous_pages=[anon])
     assert config._current_page_index == 0
@@ -1096,9 +1390,16 @@ async def test_anonymous_page(
     assert config.current_page() == anon
     button = config.button(0)
     assert button.text == "yolo"
+
     press = _on_press_callback(websocket_mock, state, config)
+
+    # We need to have a release otherwise it will be timing for a long press
+    async def press_and_release(key: int) -> None:
+        await press(mock_deck, key, key_pressed=True)
+        await press(mock_deck, key, key_pressed=False)
+
     # Click the button
-    await press(mock_deck, 0, key_pressed=True)
+    await press_and_release(0)
     # Should now be the button on the first page
     button = config.button(0)
     assert button.special_type == "go-to-page"
@@ -1107,7 +1408,7 @@ async def test_anonymous_page(
     # Click the delay button
     button = config.button(1)
     assert button.text == "foo"
-    await press(mock_deck, 1, key_pressed=True)
+    await press_and_release(1)
     # Should now still be the button because of the delay
     assert button.text == "foo"
     assert config._detached_page is not None
@@ -1120,3 +1421,133 @@ async def test_anonymous_page(
     # Should now be the button on the first page
     button = config.button(0)
     assert button.special_type == "go-to-page"
+
+    # Test load_page_as_detached and close_detached_page methods
+    assert config.current_page() == home
+    config.load_page_as_detached(anon)
+    assert config.current_page() == anon
+    config.close_detached_page()
+    assert config.current_page() == home
+
+    # Back to anon page to test that the close button works properly
+    assert config.to_page("anon") == anon
+    await press_and_release(2)  # close page button
+    assert config._detached_page is None
+    assert config.current_page() == home
+
+    # Test that to_page closes a detached page
+    config.load_page_as_detached(anon)
+    assert config.current_page() == anon
+    config.to_page(home.name)
+    assert config.current_page() == home
+    assert config._detached_page is None
+
+
+async def test_retry_logic_called_correct_number_of_times() -> None:
+    """Test retry logic in run function."""
+    # Config for the test
+    config = Config()
+
+    retry_attemps = 2
+
+    # Patch setup_ws to simulate a network failure, and patch asyncio.sleep to avoid delays
+    with (
+        patch(
+            "home_assistant_streamdeck_yaml.setup_ws",
+            side_effect=OSError("Network is down"),
+        ) as mock_setup_ws,
+        patch("asyncio.sleep", return_value=None) as mock_sleep,
+        patch("home_assistant_streamdeck_yaml.get_deck") as mock_get_deck,
+    ):
+        mock_get_deck.return_value = Mock()
+
+        # Run the function with retry_attempts = 2 to simulate retry logic
+        await run(
+            host="localhost",
+            token="",
+            protocol="ws",
+            config=config,
+            retry_attempts=retry_attemps,
+            retry_delay=0,
+        )
+
+        # Check that setup_ws was called 3 times (1 initial try + 2 retries)
+        assert mock_setup_ws.call_count == retry_attemps + 1
+
+        # Check that asyncio.sleep was called the same number of times as retries
+        assert mock_sleep.call_count == retry_attemps
+
+
+async def test_run_exits_immediately_on_zero_retries() -> None:
+    """Test that run exits immediately when retry_attempts is set to 0."""
+    config = Config()
+
+    with (
+        patch(
+            "home_assistant_streamdeck_yaml.setup_ws",
+            side_effect=OSError("Network is down"),
+        ),
+        patch("home_assistant_streamdeck_yaml.get_deck") as mock_get_deck,
+    ):
+        mock_get_deck.return_value = Mock()
+
+        # No exception should be raised, and run should return immediately
+        await run(
+            host="localhost",
+            token="",
+            protocol="ws",
+            config=config,
+            retry_attempts=0,
+            retry_delay=0,
+        )
+
+        # If setup_ws is called once, it means the retry logic did not retry
+        assert mock_get_deck.called
+
+
+def test_page_switch_clears_unused_keys(state: dict[str, dict[str, Any]]) -> None:
+    """Test that switching pages clears unused keys."""
+    # Setup pages: page1 has 2 buttons, page2 has only 1
+    page1 = Page(name="Page1", buttons=[Button(text="Btn1"), Button(text="Btn2")])
+    page2 = Page(name="Page2", buttons=[Button(text="Btn1 Only")])
+    config = Config(pages=[page1, page2])
+
+    # Patch dependencies: PILHelper for image conversion and DeviceManager to avoid needing a real deck
+    with (
+        patch(
+            "home_assistant_streamdeck_yaml.PILHelper.to_native_format",
+            return_value="mock_image_data",
+        ),
+        patch("home_assistant_streamdeck_yaml.DeviceManager") as mock_device_manager,
+    ):
+        # Configure the mock StreamDeck instance
+        mock_deck_instance = mock_device_manager().enumerate()[0]
+        mock_deck_instance.key_count.return_value = 2
+        mock_deck_instance.key_image_format.return_value = {"size": (72, 72)}
+        # Use MagicMock to easily track calls to set_key_image
+        mock_deck_instance.set_key_image = MagicMock()
+
+        # Start on the first page (page1)
+        assert config._current_page_index == 0
+
+        # Simulate switching to the second page (page2) which has fewer buttons
+        config.to_page(1)
+        assert config._current_page_index == 1
+
+        # Trigger the image update process for the current page
+        # This function should now handle clearing keys not defined on page2
+        update_all_key_images(mock_deck_instance, config, state)
+
+        # Verify that set_key_image was called correctly:
+        # - Key 0 should receive the image for the button on page2.
+        # - Key 1, which had a button on page1 but not page2, should be cleared (sent None).
+        expected_calls = [
+            call(0, "mock_image_data"),
+            call(1, None),
+        ]
+        mock_deck_instance.set_key_image.assert_has_calls(
+            expected_calls,
+            any_order=False,
+        )
+        # Ensure exactly these two calls were made for the 2-key mock deck
+        assert mock_deck_instance.set_key_image.call_count == 2  # noqa: PLR2004
